@@ -16,27 +16,30 @@ import warnings
 import parso
 from parso.python import tree
 
-from jedi._compatibility import force_unicode, is_py3
+from jedi._compatibility import force_unicode, cast_path, is_py3
 from jedi.parser_utils import get_executable_nodes
 from jedi import debug
 from jedi import settings
 from jedi import cache
+from jedi.file_io import KnownContentFileIO
 from jedi.api import classes
 from jedi.api import interpreter
 from jedi.api import helpers
 from jedi.api.completion import Completion
 from jedi.api.environment import InterpreterEnvironment
-from jedi.api.project import get_default_project
-from jedi.evaluate import Evaluator
-from jedi.evaluate import imports
-from jedi.evaluate import usages
-from jedi.evaluate.arguments import try_iter_content
-from jedi.evaluate.helpers import get_module_names, evaluate_call_of_leaf
-from jedi.evaluate.sys_path import dotted_path_in_sys_path
-from jedi.evaluate.filters import TreeNameDefinition, ParamName
-from jedi.evaluate.syntax_tree import tree_name_to_contexts
-from jedi.evaluate.context import ModuleContext
-from jedi.evaluate.context.iterable import unpack_tuple_to_dict
+from jedi.api.project import get_default_project, Project
+from jedi.inference import InferenceState
+from jedi.inference import imports
+from jedi.inference import usages
+from jedi.inference.arguments import try_iter_content
+from jedi.inference.helpers import get_module_names, infer_call_of_leaf
+from jedi.inference.sys_path import transform_path_to_dotted
+from jedi.inference.syntax_tree import tree_name_to_values
+from jedi.inference.value import ModuleValue
+from jedi.inference.base_value import ValueSet
+from jedi.inference.value.iterable import unpack_tuple_to_dict
+from jedi.inference.gradual.conversion import convert_names, convert_values
+from jedi.inference.gradual.utils import load_proper_stub_module
 
 # Jedi uses lots and lots of recursion. By setting this a little bit higher, we
 # can remove some "maximum recursion depth" errors.
@@ -58,7 +61,7 @@ class Script(object):
 
     - if `sys_path` parameter is ``None`` and ``VIRTUAL_ENV`` environment
       variable is defined, ``sys.path`` for the specified environment will be
-      guessed (see :func:`jedi.evaluate.sys_path.get_venv_path`) and used for
+      guessed (see :func:`jedi.inference.sys_path.get_venv_path`) and used for
       the script;
 
     - otherwise ``sys.path`` will match that of |jedi|.
@@ -78,10 +81,11 @@ class Script(object):
     :param sys_path: ``sys.path`` to use during analysis of the script
     :type sys_path: list
     :param environment: TODO
-    :type sys_path: Environment
+    :type environment: Environment
     """
     def __init__(self, source=None, line=None, column=None, path=None,
-                 encoding='utf-8', sys_path=None, environment=None):
+                 encoding='utf-8', sys_path=None, environment=None,
+                 _project=None):
         self._orig_path = path
         # An empty path (also empty string) should always result in no path.
         self.path = os.path.abspath(path) if path else None
@@ -97,22 +101,24 @@ class Script(object):
         if sys_path is not None and not is_py3:
             sys_path = list(map(force_unicode, sys_path))
 
-        # Load the Python grammar of the current interpreter.
-        project = get_default_project(
-            os.path.dirname(self.path)if path else os.getcwd()
-        )
+        project = _project
+        if project is None:
+            # Load the Python grammar of the current interpreter.
+            project = get_default_project(
+                os.path.dirname(self.path)if path else os.getcwd()
+            )
         # TODO deprecate and remove sys_path from the Script API.
         if sys_path is not None:
             project._sys_path = sys_path
-        self._evaluator = Evaluator(
+        self._inference_state = InferenceState(
             project, environment=environment, script_path=self.path
         )
-        self._project = project
         debug.speed('init')
-        self._module_node, source = self._evaluator.parse_and_get_code(
+        self._module_node, source = self._inference_state.parse_and_get_code(
             code=source,
             path=self.path,
             encoding=encoding,
+            use_latest_grammar=path and path.endswith('.pyi'),
             cache=False,  # No disk cache, because the current script often changes.
             diff_cache=settings.fast_parser,
             cache_path=settings.cache_directory,
@@ -137,33 +143,65 @@ class Script(object):
                              '(0-%d) for line %d (%r).' % (
                                  column, line_len, line, line_string))
         self._pos = line, column
-        self._path = path
 
         cache.clear_time_caches()
         debug.reset_time()
 
+    # Cache the module, this is mostly useful for testing, since this shouldn't
+    # be called multiple times.
+    @cache.memoize_method
     def _get_module(self):
-        name = '__main__'
+        names = None
+        is_package = False
         if self.path is not None:
-            import_names = dotted_path_in_sys_path(self._evaluator.get_sys_path(), self.path)
+            import_names, is_p = transform_path_to_dotted(
+                self._inference_state.get_sys_path(add_parent_paths=False),
+                self.path
+            )
             if import_names is not None:
-                name = '.'.join(import_names)
+                names = import_names
+                is_package = is_p
 
-        module = ModuleContext(
-            self._evaluator, self._module_node, self.path,
-            code_lines=self._code_lines
+        if self.path is None:
+            file_io = None
+        else:
+            file_io = KnownContentFileIO(cast_path(self.path), self._code)
+        if self.path is not None and self.path.endswith('.pyi'):
+            # We are in a stub file. Try to load the stub properly.
+            stub_module = load_proper_stub_module(
+                self._inference_state,
+                file_io,
+                names,
+                self._module_node
+            )
+            if stub_module is not None:
+                return stub_module
+
+        if names is None:
+            names = ('__main__',)
+
+        module = ModuleValue(
+            self._inference_state, self._module_node, file_io,
+            string_names=names,
+            code_lines=self._code_lines,
+            is_package=is_package,
         )
-        imports.add_module_to_cache(self._evaluator, name, module)
+        if names[0] not in ('builtins', '__builtin__', 'typing'):
+            # These modules are essential for Jedi, so don't overwrite them.
+            self._inference_state.module_cache.add(names, ValueSet([module]))
         return module
 
+    def _get_module_context(self):
+        return self._get_module().as_context()
+
     def __repr__(self):
         return '<%s: %s %r>' % (
             self.__class__.__name__,
             repr(self._orig_path),
-            self._evaluator.environment,
+            self._inference_state.environment,
         )
 
-    def completions(self):
+    def completions(self, fuzzy=False):
         """
         Return :class:`classes.Completion` objects. Those objects contain
         information about the completions, more than just names.
@@ -171,34 +209,14 @@ class Script(object):
         :return: Completion objects, sorted by name and __ comes last.
         :rtype: list of :class:`classes.Completion`
         """
-        debug.speed('completions start')
-        completion = Completion(
-            self._evaluator, self._get_module(), self._code_lines,
-            self._pos, self.call_signatures
-        )
-        completions = completion.completions()
+        with debug.increase_indent_cm('completions'):
+            completion = Completion(
+                self._inference_state, self._get_module_context(), self._code_lines,
+                self._pos, self.call_signatures
+            )
+            return completion.completions(fuzzy)
 
-        def iter_import_completions():
-            for c in completions:
-                tree_name = c._name.tree_name
-                if tree_name is None:
-                    continue
-                definition = tree_name.get_definition()
-                if definition is not None \
-                        and definition.type in ('import_name', 'import_from'):
-                    yield c
-
-        if len(list(iter_import_completions())) > 10:
-            # For now disable completions if there's a lot of imports that
-            # might potentially be resolved. This is the case for tensorflow
-            # and has been fixed for it. This is obviously temporary until we
-            # have a better solution.
-            self._evaluator.infer_enabled = False
-
-        debug.speed('completions end')
-        return completions
-
-    def goto_definitions(self):
+    def goto_definitions(self, **kwargs):
         """
         Return the definitions of a the path under the cursor.  goto function!
         This follows complicated paths and returns the end, not the first
@@ -208,47 +226,69 @@ class Script(object):
         because Python itself is a dynamic language, which means depending on
         an option you can have two different versions of a function.
 
+        :param only_stubs: Only return stubs for this goto call.
+        :param prefer_stubs: Prefer stubs to Python objects for this type
+            inference call.
         :rtype: list of :class:`classes.Definition`
         """
+        with debug.increase_indent_cm('goto_definitions'):
+            return self._goto_definitions(**kwargs)
+
+    def _goto_definitions(self, only_stubs=False, prefer_stubs=False):
         leaf = self._module_node.get_name_of_position(self._pos)
         if leaf is None:
             leaf = self._module_node.get_leaf_for_position(self._pos)
-            if leaf is None:
+            if leaf is None or leaf.type == 'string':
                 return []
 
-        context = self._evaluator.create_context(self._get_module(), leaf)
-        definitions = helpers.evaluate_goto_definition(self._evaluator, context, leaf)
+        context = self._get_module_context().create_context(leaf)
 
-        names = [s.name for s in definitions]
-        defs = [classes.Definition(self._evaluator, name) for name in names]
+        values = helpers.infer_goto_definition(self._inference_state, context, leaf)
+        values = convert_values(
+            values,
+            only_stubs=only_stubs,
+            prefer_stubs=prefer_stubs,
+        )
+
+        defs = [classes.Definition(self._inference_state, c.name) for c in values]
         # The additional set here allows the definitions to become unique in an
         # API sense. In the internals we want to separate more things than in
         # the API.
         return helpers.sorted_definitions(set(defs))
 
-    def goto_assignments(self, follow_imports=False, follow_builtin_imports=False):
+    def goto_assignments(self, follow_imports=False, follow_builtin_imports=False, **kwargs):
         """
         Return the first definition found, while optionally following imports.
         Multiple objects may be returned, because Python itself is a
         dynamic language, which means depending on an option you can have two
         different versions of a function.
 
+        .. note:: It is deprecated to use follow_imports and follow_builtin_imports as
+            positional arguments. Will be a keyword argument in 0.16.0.
+
         :param follow_imports: The goto call will follow imports.
         :param follow_builtin_imports: If follow_imports is True will decide if
             it follow builtin imports.
+        :param only_stubs: Only return stubs for this goto call.
+        :param prefer_stubs: Prefer stubs to Python objects for this goto call.
         :rtype: list of :class:`classes.Definition`
         """
-        def filter_follow_imports(names, check):
+        with debug.increase_indent_cm('goto_assignments'):
+            return self._goto_assignments(follow_imports, follow_builtin_imports, **kwargs)
+
+    def _goto_assignments(self, follow_imports, follow_builtin_imports,
+                          only_stubs=False, prefer_stubs=False):
+        def filter_follow_imports(names):
             for name in names:
-                if check(name):
-                    new_names = list(filter_follow_imports(name.goto(), check))
+                if name.is_import():
+                    new_names = list(filter_follow_imports(name.goto()))
                     found_builtin = False
                     if follow_builtin_imports:
                         for new_name in new_names:
                             if new_name.start_pos is None:
                                 found_builtin = True
 
-                    if found_builtin and not isinstance(name, imports.SubModuleName):
+                    if found_builtin:
                         yield name
                     else:
                         for new_name in new_names:
@@ -258,20 +298,21 @@ class Script(object):
 
         tree_name = self._module_node.get_name_of_position(self._pos)
         if tree_name is None:
-            return []
-        context = self._evaluator.create_context(self._get_module(), tree_name)
-        names = list(self._evaluator.goto(context, tree_name))
+            # Without a name we really just want to jump to the result e.g.
+            # executed by `foo()`, if we the cursor is after `)`.
+            return self.goto_definitions(only_stubs=only_stubs, prefer_stubs=prefer_stubs)
+        name = self._get_module_context().create_name(tree_name)
+        names = list(name.goto())
 
         if follow_imports:
-            def check(name):
-                return name.is_import()
-        else:
-            def check(name):
-                return isinstance(name, imports.SubModuleName)
+            names = filter_follow_imports(names)
+        names = convert_names(
+            names,
+            only_stubs=only_stubs,
+            prefer_stubs=prefer_stubs,
+        )
 
-        names = filter_follow_imports(names, check)
-
-        defs = [classes.Definition(self._evaluator, d) for d in set(names)]
+        defs = [classes.Definition(self._inference_state, d) for d in set(names)]
         return helpers.sorted_definitions(defs)
 
     def usages(self, additional_module_paths=(), **kwargs):
@@ -301,9 +342,9 @@ class Script(object):
                 # Must be syntax
                 return []
 
-            names = usages.usages(self._get_module(), tree_name)
+            names = usages.usages(self._get_module_context(), tree_name)
 
-            definitions = [classes.Definition(self._evaluator, n) for n in names]
+            definitions = [classes.Definition(self._inference_state, n) for n in names]
             if not include_builtins:
                 definitions = [d for d in definitions if not d.in_builtin_module()]
             return helpers.sorted_definitions(definitions)
@@ -325,40 +366,35 @@ class Script(object):
 
         :rtype: list of :class:`classes.CallSignature`
         """
-        call_signature_details = \
-            helpers.get_call_signature_details(self._module_node, self._pos)
-        if call_signature_details is None:
+        call_details = helpers.get_call_signature_details(self._module_node, self._pos)
+        if call_details is None:
             return []
 
-        context = self._evaluator.create_context(
-            self._get_module(),
-            call_signature_details.bracket_leaf
-        )
+        context = self._get_module_context().create_context(call_details.bracket_leaf)
         definitions = helpers.cache_call_signatures(
-            self._evaluator,
+            self._inference_state,
             context,
-            call_signature_details.bracket_leaf,
+            call_details.bracket_leaf,
             self._code_lines,
             self._pos
         )
         debug.speed('func_call followed')
 
-        return [classes.CallSignature(self._evaluator, d.name,
-                                      call_signature_details.bracket_leaf.start_pos,
-                                      call_signature_details.call_index,
-                                      call_signature_details.keyword_name_str)
-                for d in definitions if hasattr(d, 'py__call__')]
+        # TODO here we use stubs instead of the actual values. We should use
+        # the signatures from stubs, but the actual values, probably?!
+        return [classes.CallSignature(self._inference_state, signature, call_details)
+                for signature in definitions.get_signatures()]
 
     def _analysis(self):
-        self._evaluator.is_analysis = True
-        self._evaluator.analysis_modules = [self._module_node]
-        module = self._get_module()
+        self._inference_state.is_analysis = True
+        self._inference_state.analysis_modules = [self._module_node]
+        module = self._get_module_context()
         try:
             for node in get_executable_nodes(self._module_node):
                 context = module.create_context(node)
                 if node.type in ('funcdef', 'classdef'):
                     # Resolve the decorators.
-                    tree_name_to_contexts(self._evaluator, context, node.children[1])
+                    tree_name_to_values(self._inference_state, context, node.children[1])
                 elif isinstance(node, tree.Import):
                     import_names = set(node.get_defined_names())
                     if node.is_nested():
@@ -366,22 +402,22 @@ class Script(object):
                     for n in import_names:
                         imports.infer_import(context, n)
                 elif node.type == 'expr_stmt':
-                    types = context.eval_node(node)
+                    types = context.infer_node(node)
                     for testlist in node.children[:-1:2]:
                         # Iterate tuples.
                         unpack_tuple_to_dict(context, types, testlist)
                 else:
                     if node.type == 'name':
-                        defs = self._evaluator.goto_definitions(context, node)
+                        defs = self._inference_state.goto_definitions(context, node)
                     else:
-                        defs = evaluate_call_of_leaf(context, node)
+                        defs = infer_call_of_leaf(context, node)
                     try_iter_content(defs)
-                self._evaluator.reset_recursion_limitations()
+                self._inference_state.reset_recursion_limitations()
 
-            ana = [a for a in self._evaluator.analysis if self.path == a.path]
+            ana = [a for a in self._inference_state.analysis if self.path == a.path]
             return sorted(set(ana), key=lambda x: x.line)
         finally:
-            self._evaluator.is_analysis = False
+            self._inference_state.is_analysis = False
 
 
 class Interpreter(Script):
@@ -399,6 +435,7 @@ class Interpreter(Script):
     >>> print(script.completions()[0].name)
     upper
     """
+    _allow_descriptor_getattr_default = True
 
     def __init__(self, source, namespaces, **kwds):
         """
@@ -426,17 +463,23 @@ class Interpreter(Script):
             if not isinstance(environment, InterpreterEnvironment):
                 raise TypeError("The environment needs to be an InterpreterEnvironment subclass.")
 
-        super(Interpreter, self).__init__(source, environment=environment, **kwds)
+        super(Interpreter, self).__init__(source, environment=environment,
+                                          _project=Project(os.getcwd()), **kwds)
         self.namespaces = namespaces
+        self._inference_state.allow_descriptor_getattr = self._allow_descriptor_getattr_default
 
-    def _get_module(self):
-        return interpreter.MixedModuleContext(
-            self._evaluator,
-            self._module_node,
-            self.namespaces,
-            path=self.path,
+    @cache.memoize_method
+    def _get_module_context(self):
+        tree_module_value = ModuleValue(
+            self._inference_state, self._module_node,
+            file_io=KnownContentFileIO(self.path, self._code),
+            string_names=('__main__',),
             code_lines=self._code_lines,
         )
+        return interpreter.MixedModuleContext(
+            tree_module_value,
+            self.namespaces,
+        )
 
 
 def names(source=None, path=None, encoding='utf-8', all_scopes=False,
@@ -459,24 +502,13 @@ def names(source=None, path=None, encoding='utf-8', all_scopes=False,
         is_def = _def._name.tree_name.is_definition()
         return definitions and is_def or references and not is_def
 
-    def create_name(name):
-        if name.parent.type == 'param':
-            cls = ParamName
-        else:
-            cls = TreeNameDefinition
-        is_module = name.parent.type == 'file_input'
-        return cls(
-            module_context.create_context(name if is_module else name.parent),
-            name
-        )
-
     # Set line/column to a random position, because they don't matter.
     script = Script(source, line=1, column=0, path=path, encoding=encoding, environment=environment)
-    module_context = script._get_module()
+    module_context = script._get_module_context()
     defs = [
         classes.Definition(
-            script._evaluator,
-            create_name(name)
+            script._inference_state,
+            module_context.create_name(name)
         ) for name in get_module_names(script._module_node, all_scopes)
     ]
     return sorted(filter(def_ref_filter, defs), key=lambda x: (x.line, x.column))
