@@ -1,6 +1,4 @@
-# todo probably remove test_integration_keyword
-
-def test_keyword_doc():
+def test_keyword_doc(Script):
     r = list(Script("or", 1, 1).goto_definitions())
     assert len(r) == 1
     assert len(r[0].doc) > 100
@@ -14,17 +12,18 @@ def test_keyword_doc():
     assert k.doc.startswith(imp_start)
 
 
-def test_blablabla():
+def test_blablabla(Script):
     defs = Script("import").goto_definitions()
     assert len(defs) == 1 and [1 for d in defs if d.doc]
     # unrelated to #44
 
 
-def test_operator_doc(self):
+def test_operator_doc(Script):
     r = list(Script("a == b", 1, 3).goto_definitions())
     assert len(r) == 1
     assert len(r[0].doc) > 100
 
-def test_lambda():
+
+def test_lambda(Script):
     defs = Script('lambda x: x', column=0).goto_definitions()
     assert [d.type for d in defs] == ['keyword']
