@@ -16,8 +16,18 @@ class Y(X):
     #? ['func']
     def f
 
+    #? ['__doc__']
+    __doc__
     #? []
+    def __doc__
+
+    # This might or might not be what we wanted, currently properties are also
+    # used like this. IMO this is not wanted ~dave.
+    #? ['__class__']
     def __class__
+    #? []
+    __class__
+
 
     #? ['__repr__']
     def __repr__
