@@ -22,14 +22,20 @@ cd = e
 
 #! ['module math']
 import math
-#! ['import math']
+#! ['module math']
 math
 
-#! ['import math']
+#! ['module math']
 b = math
 #! ['b = math']
 b
 
+#! 18 ['foo = 10']
+foo = 10;print(foo)
+
+# -----------------
+# classes
+# -----------------
 class C(object):
     def b(self):
         #! ['b = math']
@@ -38,7 +44,7 @@ class C(object):
         self.b
         #! 14 ['def b']
         self.b()
-        #! 11 ['self']
+        #! 11 ['param self']
         self.b
         return 1
 
@@ -93,9 +99,33 @@ ClassVar().x = ''
 
 # Recurring use of the same var name, github #315
 def f(t=None):
-    #! 9 ['t=None']
+    #! 9 ['param t=None']
     t = t or 1
 
+
+class X():
+    pass
+
+#! 3 []
+X(foo=x)
+
+
+# Multiple inheritance
+class Foo:
+    def foo(self):
+        print("foo")
+class Bar:
+    def bar(self):
+        print("bar")
+class Baz(Foo, Bar):
+    def baz(self):
+        #! ['def foo']
+        super().foo
+        #! ['def bar']
+        super().bar
+        #! ['instance Foo']
+        super()
+
 # -----------------
 # imports
 # -----------------
@@ -107,6 +137,8 @@ import_tree.a
 
 #! ['module mod1']
 import import_tree.mod1
+#! ['module mod1']
+from import_tree.mod1
 #! ['a = 1']
 import_tree.mod1.a
 
@@ -135,7 +167,7 @@ mod1.a
 #! ['a = 1.0']
 from import_tree.pkg.mod1 import a
 
-#! ['import os']
+#! ['module os']
 from .imports import os
 
 #! ['some_variable = 1']
@@ -167,9 +199,9 @@ class ClassDef():
 # -----------------
 
 param = ClassDef
-#! 8 ['param']
+#! 8 ['param param']
 def ab1(param): pass
-#! 9 ['param']
+#! 9 ['param param']
 def ab2(param): pass
 #! 11 ['param = ClassDef']
 def ab3(a=param): pass
@@ -188,9 +220,10 @@ for key, value in [(1,2)]:
     #! ['for key, value in [(1,2)]: key']
     key
 
-for i in []:
-    #! ['for i in []: i']
-    i
+#! 4 ['for y in [1]: y']
+for y in [1]:
+    #! ['for y in [1]: y']
+    y
 
 # -----------------
 # decorator
@@ -198,7 +231,7 @@ for i in []:
 def dec(dec_param=3):
     pass
 
-#! 8 ['dec_param=3']
+#! 8 ['param dec_param=3']
 @dec(dec_param=5)
 def y():
     pass
