@@ -24,7 +24,7 @@ def function_parameters(a: A, b, c: str, d: int, e: str, f: str, g: int=4):
     d
     #? str()
     e
-    #? int() str()
+    #? str()
     f
     # int()
     g
@@ -44,7 +44,7 @@ def return_none() -> None:
     """
     pass
 
-#?
+#? None
 return_none()
 
 
@@ -68,7 +68,7 @@ def return_annotation_and_docstring() -> str:
     """
     pass
 
-#? str() int()
+#? str()
 return_annotation_and_docstring()
 
 
@@ -138,7 +138,7 @@ function_with_non_pep_0484_annotation(1, 2, 3, "force string")
 def function_forward_reference_dynamic(
         x: return_str_type(),
         y: "return_str_type()") -> None:
-    #?
+    #? str()
     x
     #? str()
     y
@@ -165,3 +165,19 @@ def keyword_only(a: str, *, b: str):
     a.startswi
     #? ['startswith']
     b.startswi
+
+
+def argskwargs(*args: int, **kwargs: float):
+    """
+    This might be a bit confusing, but is part of the standard.
+    args is changed to Tuple[int] in this case and kwargs to Dict[str, float],
+    which makes sense if you think about it a bit.
+    """
+    #? tuple()
+    args
+    #? int()
+    args[0]
+    #? str()
+    next(iter(kwargs.keys()))
+    #? float()
+    kwargs['']
