@@ -18,7 +18,7 @@ How to run tests?
 +++++++++++++++++
 
 Jedi uses pytest_ to run unit and integration tests.  To run tests,
-simply run ``py.test``.  You can also use tox_ to run tests for
+simply run ``pytest``.  You can also use tox_ to run tests for
 multiple Python versions.
 
 .. _pytest: http://pytest.org
@@ -30,7 +30,7 @@ definitions), ``#!`` (assignments), or ``#<`` (usages).
 There is also support for third party libraries. In a normal test run they are
 not being executed, you have to provide a ``--thirdparty`` option.
 
-In addition to standard `-k` and `-m` options in py.test, you can use
+In addition to standard `-k` and `-m` options in pytest, you can use the
 `-T` (`--test-files`) option to specify integration test cases to run.
 It takes the format of ``FILE_NAME[:LINE[,LINE[,...]]]`` where
 ``FILE_NAME`` is a file in ``test/completion`` and ``LINE`` is a line
@@ -38,20 +38,20 @@ number of the test comment.  Here is some recipes:
 
 Run tests only in ``basic.py`` and ``imports.py``::
 
-    py.test test/test_integration.py -T basic.py -T imports.py
+    pytest test/test_integration.py -T basic.py -T imports.py
 
 Run test at line 4, 6, and 8 in ``basic.py``::
 
-    py.test test/test_integration.py -T basic.py:4,6,8
+    pytest test/test_integration.py -T basic.py:4,6,8
 
-See ``py.test --help`` for more information.
+See ``pytest --help`` for more information.
 
 If you want to debug a test, just use the ``--pdb`` option.
 
 Alternate Test Runner
 +++++++++++++++++++++
 
-If you don't like the output of ``py.test``, there's an alternate test runner
+If you don't like the output of ``pytest``, there's an alternate test runner
 that you can start by running ``./run.py``. The above example could be run by::
 
     ./run.py basic 4 6 8 50-80
@@ -123,9 +123,11 @@ import jedi
 from jedi import debug
 from jedi._compatibility import unicode, is_py3
 from jedi.api.classes import Definition
-from jedi.api.completion import get_user_scope
+from jedi.api.completion import get_user_context
 from jedi import parser_utils
 from jedi.api.environment import get_default_environment, get_system_environment
+from jedi.inference.gradual.conversion import convert_values
+from jedi.inference.analysis import Warning
 
 
 TEST_COMPLETIONS = 0
@@ -137,16 +139,8 @@ TEST_USAGES = 3
 grammar36 = parso.load_grammar(version='3.6')
 
 
-class IntegrationTestCase(object):
-    def __init__(self, test_type, correct, line_nr, column, start, line,
-                 path=None, skip_version_info=None):
-        self.test_type = test_type
-        self.correct = correct
-        self.line_nr = line_nr
-        self.column = column
-        self.start = start
-        self.line = line
-        self.path = path
+class BaseTestCase(object):
+    def __init__(self, skip_version_info=None):
         self._skip_version_info = skip_version_info
         self._skip = None
 
@@ -174,6 +168,19 @@ class IntegrationTestCase(object):
                 operator_, min_version[0], min_version[1]
             )
 
+
+class IntegrationTestCase(BaseTestCase):
+    def __init__(self, test_type, correct, line_nr, column, start, line,
+                 path=None, skip_version_info=None):
+        super(IntegrationTestCase, self).__init__(skip_version_info)
+        self.test_type = test_type
+        self.correct = correct
+        self.line_nr = line_nr
+        self.column = column
+        self.start = start
+        self.line = line
+        self.path = path
+
     @property
     def module_name(self):
         return os.path.splitext(os.path.basename(self.path))[0]
@@ -211,7 +218,7 @@ class IntegrationTestCase(object):
 
     def run_goto_definitions(self, compare_cb, environment):
         script = self.script(environment)
-        evaluator = script._evaluator
+        inference_state = script._inference_state
 
         def comparison(definition):
             suffix = '()' if definition.type == 'instance' else ''
@@ -223,19 +230,19 @@ class IntegrationTestCase(object):
                 string = match.group(0)
                 parser = grammar36.parse(string, start_symbol='eval_input', error_recovery=False)
                 parser_utils.move(parser.get_root_node(), self.line_nr)
-                element = parser.get_root_node()
-                module_context = script._get_module()
-                # The context shouldn't matter for the test results.
-                user_context = get_user_scope(module_context, (self.line_nr, 0))
-                if user_context.api_type == 'function':
-                    user_context = user_context.get_function_execution()
-                element.parent = user_context.tree_node
-                results = evaluator.eval_element(user_context, element)
+                node = parser.get_root_node()
+                module_context = script._get_module_context()
+                user_context = get_user_context(module_context, (self.line_nr, 0))
+                # TODO needed?
+                #if user_context._value.api_type == 'function':
+                #    user_context = user_context.get_function_execution()
+                node.parent = user_context.tree_node
+                results = convert_values(user_context.infer_node(node))
                 if not results:
                     raise Exception('Could not resolve %s on line %s'
                                     % (match.string, self.line_nr - 1))
 
-                should_be |= set(Definition(evaluator, r.name) for r in results)
+                should_be |= set(Definition(inference_state, r.name) for r in results)
             debug.dbg('Finished getting types', color='YELLOW')
 
             # Because the objects have different ids, `repr`, then compare.
@@ -255,7 +262,10 @@ class IntegrationTestCase(object):
     def run_usages(self, compare_cb, environment):
         result = self.script(environment).usages()
         self.correct = self.correct.strip()
-        compare = sorted((r.module_name, r.line, r.column) for r in result)
+        compare = sorted(
+            (re.sub(r'^test\.completion\.', '', r.module_name), r.line, r.column)
+            for r in result
+        )
         wanted = []
         if not self.correct:
             positions = []
@@ -274,6 +284,47 @@ class IntegrationTestCase(object):
         return compare_cb(self, compare, sorted(wanted))
 
 
+class StaticAnalysisCase(BaseTestCase):
+    """
+    Static Analysis cases lie in the static_analysis folder.
+    The tests also start with `#!`, like the goto_definition tests.
+    """
+    def __init__(self, path):
+        self._path = path
+        self.name = os.path.basename(path)
+        with open(path) as f:
+            self._source = f.read()
+
+        skip_version_info = None
+        for line in self._source.splitlines():
+            skip_version_info = skip_python_version(line) or skip_version_info
+
+        super(StaticAnalysisCase, self).__init__(skip_version_info)
+
+    def collect_comparison(self):
+        cases = []
+        for line_nr, line in enumerate(self._source.splitlines(), 1):
+            match = re.match(r'(\s*)#! (\d+ )?(.*)$', line)
+            if match is not None:
+                column = int(match.group(2) or 0) + len(match.group(1))
+                cases.append((line_nr + 1, column, match.group(3)))
+        return cases
+
+    def run(self, compare_cb, environment):
+        analysis = jedi.Script(
+            self._source,
+            path=self._path,
+            environment=environment,
+        )._analysis()
+        typ_str = lambda inst: 'warning ' if isinstance(inst, Warning) else ''
+        analysis = [(r.line, r.column, typ_str(r) + r.name)
+                    for r in analysis]
+        compare_cb(self, analysis, self.collect_comparison())
+
+    def __repr__(self):
+        return "<%s: %s>" % (self.__class__.__name__, os.path.basename(self._path))
+
+
 def skip_python_version(line):
     # check for python minimal version number
     match = re.match(r" *# *python *([<>]=?|==) *(\d+(?:\.\d+)?)$", line)
@@ -295,7 +346,7 @@ def collect_file_tests(path, lines, lines_to_execute):
     skip_version_info = None
     for line_nr, line in enumerate(lines, 1):
         if correct is not None:
-            r = re.match('^(\d+)\s*(.*)$', correct)
+            r = re.match(r'^(\d+)\s*(.*)$', correct)
             if r:
                 column = int(r.group(1))
                 correct = r.group(2)
@@ -353,9 +404,11 @@ def collect_dir_tests(base_dir, test_files, check_thirdparty=False):
             path = os.path.join(base_dir, f_name)
 
             if is_py3:
-                source = open(path, encoding='utf-8').read()
+                with open(path, encoding='utf-8') as f:
+                    source = f.read()
             else:
-                source = unicode(open(path).read(), 'UTF-8')
+                with open(path) as f:
+                    source = unicode(f.read(), 'UTF-8')
 
             for case in collect_file_tests(path, StringIO(source),
                                            lines_to_execute):
@@ -396,7 +449,7 @@ if __name__ == '__main__':
     test_files = {}
     last = None
     for arg in arguments['<rest>']:
-        match = re.match('(\d+)-(\d+)', arg)
+        match = re.match(r'(\d+)-(\d+)', arg)
         if match:
             start, end = match.groups()
             test_files[last].append((int(start), int(end)))
@@ -412,7 +465,6 @@ if __name__ == '__main__':
     dir_ = os.path.dirname(os.path.realpath(__file__))
     completion_test_dir = os.path.join(dir_, '../test/completion')
     completion_test_dir = os.path.abspath(completion_test_dir)
-    summary = []
     tests_fail = 0
 
     # execute tests
@@ -422,9 +474,11 @@ if __name__ == '__main__':
         cases += collect_dir_tests(completion_test_dir, test_files, True)
 
     def file_change(current, tests, fails):
-        if current is not None:
+        if current is None:
+            current = ''
+        else:
             current = os.path.basename(current)
-        print('%s \t\t %s tests and %s fails.' % (current, tests, fails))
+        print('{:25} {} tests and {} fails.'.format(current, tests, fails))
 
     def report(case, actual, desired):
         if actual == desired:
@@ -470,8 +524,6 @@ if __name__ == '__main__':
 
     print('\nSummary: (%s fails of %s tests) in %.3fs'
           % (tests_fail, len(cases), time.time() - t_start))
-    for s in summary:
-        print(s)
 
     exit_code = 1 if tests_fail else 0
     sys.exit(exit_code)
