@@ -7,21 +7,21 @@ from ..helpers import cwd_at
 def test_import_empty(Script):
     """ github #340, return the full word. """
     completion = Script("import ").completions()[0]
-    definition = completion.follow_definition()[0]
+    definition = completion.infer()[0]
     assert definition
 
 
 def check_follow_definition_types(Script, source):
     # nested import
     completions = Script(source, path='some_path.py').completions()
-    defs = chain.from_iterable(c.follow_definition() for c in completions)
+    defs = chain.from_iterable(c.infer() for c in completions)
     return [d.type for d in defs]
 
 
-def test_follow_import_incomplete(Script):
+def test_follow_import_incomplete(Script, environment):
     """
     Completion on incomplete imports should always take the full completion
-    to do any evaluation.
+    to do any type inference.
     """
     datetime = check_follow_definition_types(Script, "import itertool")
     assert datetime == ['module']
@@ -30,15 +30,17 @@ def test_follow_import_incomplete(Script):
     itert = jedi.Script("from itertools import ").completions()
     definitions = [d for d in itert if d.name == 'chain']
     assert len(definitions) == 1
-    assert [d.type for d in definitions[0].follow_definition()] == ['class']
+    assert [d.type for d in definitions[0].infer()] == ['class']
 
     # incomplete `from * import` part
     datetime = check_follow_definition_types(Script, "from datetime import datetim")
-    assert set(datetime) == {'class', 'instance'}  # py33: builtin and pure py version
-
+    if environment.version_info.major == 2:
+        assert datetime == ['class']
+    else:
+        assert set(datetime) == {'class', 'instance'}  # py3: builtin and pure py version
     # os.path check
     ospath = check_follow_definition_types(Script, "from os.path import abspat")
-    assert ospath == ['function']
+    assert set(ospath) == {'function'}
 
     # alias
     alias = check_follow_definition_types(Script, "import io as abcd; abcd")
