@@ -3,7 +3,7 @@ import os
 import pytest
 
 from jedi import api
-from jedi.evaluate import imports
+from jedi.inference import imports
 from ..helpers import cwd_at
 
 
@@ -17,7 +17,18 @@ def test_add_dynamic_mods(Script):
     # Other fictional modules in another place in the fs.
     src2 = 'from .. import setup; setup.r(1)'
     script = Script(src1, path='../setup.py')
-    imports.load_module(script._evaluator, os.path.abspath(fname), src2)
+    imports.load_module(script._inference_state, os.path.abspath(fname), src2)
     result = script.goto_definitions()
     assert len(result) == 1
     assert result[0].description == 'class int'
+
+
+def test_add_bracket_after_function(monkeypatch, Script):
+    settings = api.settings
+    monkeypatch.setattr(settings, 'add_bracket_after_function', True)
+    script = Script('''\
+def foo():
+    pass
+foo''')
+    completions = script.completions()
+    assert completions[0].complete == '('
