@@ -8,7 +8,7 @@ def test_exclude_builtin_modules(Script):
         return [(d.line, d.column) for d in Script(source, column=8).usages(include_builtins=include)]
     source = '''import sys\nprint(sys.path)'''
     places = get(include=True)
-    assert places == [(None, None), (1, 7), (2, 6)]
+    assert len(places) > 2  # Includes stubs
 
     places = get(include=False)
     assert places == [(1, 7), (2, 6)]
