@@ -76,7 +76,15 @@ class TestSetupReadline(unittest.TestCase):
         goal = {s + el for el in dir(os)}
         # There are minor differences, e.g. the dir doesn't include deleted
         # items as well as items that are not only available on linux.
-        assert len(set(self.completions(s)).symmetric_difference(goal)) < 20
+        difference = set(self.completions(s)).symmetric_difference(goal)
+        difference = {
+            x for x in difference
+            if all(not x.startswith('from os import ' + s)
+                   for s in ['_', 'O_', 'EX_', 'MFD_', 'SF_', 'ST_'])
+        }
+        # There are quite a few differences, because both Windows and Linux
+        # (posix and nt) librariesare included.
+        assert len(difference) < 20
 
     @cwd_at('test')
     def test_local_import(self):
